;file lib_uartCalculations.inc

;-----------------------------------------------------------------------------------------
; uart calculations (also sets int_period)
;-----------------------------------------------------------------------------------------

if maxbaud < 1200
  maxbaud = 1200
endif
;calculate uartfs
uartfs = maxbaud * 2		;2x oversampling at maxbaud
if uartfs < cpufreq / 240	;limit int_period to 240
  uartfs = cpufreq / 240
endif
expand
;Calculations are for no RTCC prescaler
noexpand
if uartfs <= 3600
expand
uartfs = 3600
noexpand
else
  if uartfs <= 7200
expand
uartfs = 7200
noexpand
  else
    if uartfs <= 14400
expand
uartfs = 14400
noexpand
    else
      if uartfs <= 28800
expand
uartfs = 28800
noexpand
      else
        if uartfs <= 57600
expand
uartfs = 57600
noexpand
        else
          if uartfs <= 115200
expand
uartfs = 115200
noexpand
          else
            if uartfs <= 230400
expand
uartfs = 230400
noexpand
            else
              if uartfs <= 460800
expand
uartfs = 460800
noexpand
              else
uartfs = 921600
noexpand
              endif
            endif
          endif
        endif
      endif
    endif
  endif
endif
expand
int_period = ((2 * cpufreq / uartfs) + 1) / 2		;int_period = int(cpufreq/uartfs + 0.5)
noexpand

mscount		equ	((2*1000*cpufreq/1_000_000/int_period)+1)/2	;isr cycles for 1 msec
			;50MHz -> 230, 20MHz -> 115, 10MHz -> 57, 4MHz -> 29
uscount		equ	((2*250*cpufreq/1_000_000/int_period)+1)/2	;isr cycles for 250 usec
			;50MHz -> 58, 20MHz -> 29, 10MHz -> 14, 4MHz -> 7

;The following baudrates are available

_lib_uart_calc3	macro	baud, ratio, divide
if ratio == 1
expand
BAUD_??baud??_1 = divide
noexpand
endif
if ratio == 2
expand
BAUD_??baud??_2 = divide
noexpand
endif
if ratio == 3
expand
BAUD_??baud??_3 = divide
noexpand
endif
if ratio == 4
expand
BAUD_??baud??_4 = divide
noexpand
endif
if ratio == 5
expand
BAUD_??baud??_5 = divide
noexpand
endif
if ratio == 6
expand
BAUD_??baud??_6 = divide
noexpand
endif
if ratio == 7
expand
BAUD_??baud??_7 = divide
noexpand
endif
if ratio == 8
expand
BAUD_??baud??_8 = divide
noexpand
endif
if ratio == 9
expand
BAUD_??baud??_9 = divide
noexpand
endif
if ratio == 10
expand
BAUD_??baud??_10 = divide
noexpand
endif
if ratio == 11
expand
BAUD_??baud??_11 = divide
noexpand
endif
if ratio == 12
expand
BAUD_??baud??_12 = divide
noexpand
endif
if ratio == 13
expand
BAUD_??baud??_13 = divide
noexpand
endif
if ratio == 14
expand
BAUD_??baud??_14 = divide
noexpand
endif
if ratio == 15
expand
BAUD_??baud??_15 = divide
noexpand
endif
if ratio == 16
expand
BAUD_??baud??_16 = divide
noexpand
endif
endm

_lib_uart_calc2	macro	baud
if ratio < THREADS
  ratio = ratio + 1
  divide = ((2 * uartfs / baud / ratio) + 1) / 2	;defines bittime, roundoff value
  if (THREADS / ratio) * ratio == THREADS		;ratio must be integer multiple of THREADS
    ;minimum divide must be 2
    ;to sample at middle of databits, startdelay must be 1.5 * bittime
    if divide < 2		;min startdelay = 1.5 * 2 = 3
      divide = 0
    endif
    if divide > 170		;max startdelay = 1.5 * 170 = 255
      divide = 0
    endif
    if divide != 0
      _lib_uart_calc3 baud,ratio,divide
    endif
  endif
endif
endm

lib_uart_calc	macro	baud
ratio = 0
if baud <= maxbaud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
  _lib_uart_calc2 baud
endif
endm

;serial data formats
E71		equ	%00001110	; even parity, 7 databits, 1 stopbit , 1 startbit, 10 bits total
E72		equ	%00001111	; even parity, 7 databits, 2 stopbit , 1 startbit, 11 bits total
E81		equ	%00001011	; even parity, 8 databits, 1 stopbit , 1 startbit, 11 bits total
O71		equ	%00010110	; odd  parity, 7 databits, 1 stopbit , 1 startbit, 10 bits total
O72		equ	%00010111	; odd  parity, 7 databits, 2 stopbit , 1 startbit, 11 bits total
O81		equ	%00010011	; odd  parity, 8 databits, 1 stopbit , 1 startbit, 11 bits total
N71		equ	%00000101	; no   parity, 7 databits, 1 stopbit , 1 startbit,  9 bits total 
N72		equ	%00000110	; no   parity, 7 databits, 2 stopbit , 1 startbit, 10 bits total 
N81		equ	%00000010	; no   parity, 8 databits, 1 stopbit , 1 startbit, 10 bits total
N82		equ	%00000011	; no   parity, 8 databits, 2 stopbit , 1 startbit, 11 bits total
UART_ENABLE	equ	%01000000	; load [UART_ENABLE] + dataformat value into rx_format or tx_format

;end of file lib_uartCalculations.inc
